# -*- coding: utf_8 -*-
import hashlib

from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.shortcuts import render, redirect
from django.template import RequestContext


from rest_framework import viewsets

from fotos.models import Tag, Picture
from fotos.serializers import UserSerializer, TagSerializer, PictureSerializer


class UserViewSet(viewsets.ModelViewSet):
    queryset = User.objects.all().order_by('-date_joined')
    serializer_class = UserSerializer


class TagViewSet(viewsets.ModelViewSet):
    queryset = Tag.objects.all()
    serializer_class = TagSerializer


class PictureViewSet(viewsets.ModelViewSet):
    queryset = Picture.objects.all().order_by('-pk')
    serializer_class = PictureSerializer

    # http://www.django-rest-framework.org/tutorial/4-authentication-and-permissions/#associating-snippets-with-users
    def perform_create(self, serializer):
        username = self.request.data['username']
        password = self.request.data['password']

        try:
            user = User.objects.get(username=username)
            serializer.save(user=user)
        except ObjectDoesNotExist:
            user = User.objects.create(username=username, email=username)
            user.password = password
            user.save()
            serializer.save(user=user)


#
# INTERFACE
#

def home(request):
    if request.user.is_authenticated():
        return redirect('user', username=request.user.username)

    return render(request, 'fotos/index.html', context_instance=RequestContext(request))

def sign_in(request):
    username = request.POST['username']
    password = request.POST['password']

    user = authenticate(username=username, password=password)

    if user is not None:
        if user.is_active:
            login(request, user)
            return redirect('user', username=username)
        else:
            #messages.add_message(request, messages.WARNING, 'Conta desabilitada.')
            return redirect('home')
    else:
        #messages.add_message(request, messages.WARNING, 'Usuário ou senha inválidos.')
        return redirect('home')

def sign_out(request):
    logout(request)
    # Redirect to a success page.    
    return redirect('home')

def user(request, username):
    try:
        u = User.objects.get(username=username)
    except User.DoesNotExist:
        raise Http404("Não existe usuário com esse nome.")
    gravatar = hashlib.md5(u.email.lower()).hexdigest()
    pictures = Picture.objects.filter(user=u).order_by("-pub_date")

    return render(request, 'fotos/user.html', locals())

def picture(request, pic_uuid):
    picture = Picture.objects.get(uuid=pic_uuid)
    gravatar = hashlib.md5(picture.user.email.lower()).hexdigest()

    return render(request, 'fotos/picture.html', locals())

@login_required(login_url='/')
def delete(request, pic_uuid):
    try:
        picture = Picture.objects.get(uuid=pic_uuid)
    except Picture.DoesNotExist:
        raise Http404("Essa foto não existe.")

    if request.user == picture.user:
        picture.delete()
        messages.add_message(request, messages.SUCCESS, 'Foto excluída com sucesso.')

        return redirect('user', username=request.user.username)
    else:
        messages.add_message(request, messages.WARNING, 'Usuário não tem permissão para excluir foto.')

        return redirect('user', username=request.user.username)
